# frozen_string_literal: true

#--
# wwwjdic
# © 2014-2021 Marco Bresciani
#
# This file is part of wwwjdic.
#
# wwwjdic is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# wwwjdic is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License
# along with wwwjdic. If not, see <http://www.gnu.org/licenses/>.
#
# SPDX-FileCopyrightText: 2014-2021 Marco Bresciani
#
# SPDX-License-Identifier: GPL-3.0-or-later
#++

module WWWJDic
  # Author:: {Marco Bresciani}[mailto:marcobresciani_1974@libero.it]
  # Copyright:: © 2014-2021 Marco Bresciani
  # License:: GNU General Public License version 3

  # Allowed parameters for configuration
  ALLOWED_PARAMS = %i[dict display key server].freeze

  # All parameters for configuration
  ALL_PARAMS = ALLOWED_PARAMS + [:search]

  # Numeric codes for language-specific dictionaries
  AVAIL_LANGS = { en: '1', de: 'G', fr: 'H', ru: 'I', sv: 'J', hu: 'K', es: 'L', nl: 'M', sl: 'N', it: 'O' }.freeze

  # Numeric codes for dictionaries usage
  DICTIONARY_CODES = [('1'..'9').to_a, ('A'..'R').to_a].flatten

  # Naming for all supported dictionaries
  DICTIONARY_NAMES = ['Jpn-Eng General (EDICT)',
                      'Japanese Names (ENAMDICT)',
                      'Computing/Telecomms',
                      'Life Sciences/Bio-Med',
                      'Legal Terms',
                      'Finance/Marketing',
                      'Buddhism',
                      'Miscellaneous',
                      'Special Text-glossing',
                      'Engineering/Science',
                      'Linguistics',
                      'River & Water Systems',
                      'Automobile Industry',
                      'Japanese Wordnet',
                      'Work-in-progress File',
                      'Japanese-German (WaDoku)',
                      'Japanese-French',
                      'Japanese-Russian',
                      'Japanese-Swedish',
                      'Japanese-Hungarian',
                      'Japanese-Spanish',
                      'Japanese-Dutch',
                      'Japanese-Slovenian',
                      'Japanese-Italian',
                      'Untranslated',
                      'Combined Jpn-Eng',
                      'Expanded Text-glossing'].freeze

  # Mapping between dictionaries names with codes
  DICTS_BY_CODES = DICTIONARY_CODES.zip(DICTIONARY_NAMES).to_h

  # Mapping between dictionaries codes with names
  DICTS_BY_NAMES = DICTIONARY_NAMES.zip(DICTIONARY_CODES).to_h

  # Display modes
  DISPLAY = { regular: 'M', raw: 'Z' }.freeze

  # k is the key type:
  #
  #     for dictionary lookups
  #         for English keys use E, or P to get just "common words", Q
  # to get an "exact match" and R to get both;
  #         for Japanese keys use J (this is mandatory for romaji keys),
  # P to get just "common words" (doesn't work with romaji), K for keys
  # starting with kanji (first position), and L for kanji keys in any
  # position.
  #     for kanji lookups, use M followed by the KANJIDIC letter codes
  # (B, U, V, N, etc.) or J if a reading or kanji is being provided. An
  # optional stroke-count or stroke-count range can be included by
  # placing it between "=" characters;
  #     for text glossing use G, or H to turn on the "no repeated
  # translations" option.
  #     for multi-radical kanji lookups use J for jouyou kanji-only, H
  # to include JIS X 0212 kanji, and X for anything else. An optional
  # stroke-count or stroke-count range can be included by placing it
  # between "=" characters.
  #     for example sentence lookups with indexed words use E for EUC,
  # ISO-2022-JP or UCS, S for Shift_JIS and U for UTF-8, followed by
  # "lookupword=n=kana=". The kana is optional and is there to
  # disambiguate between different headwords. For "n", 1 => random
  # selection of 10, anything else => display up to 100 sentences
  # starting at n.
  #     for example sentence lookups using a regular expression, use E
  # for EUC, ISO-2022-JP or UCS, S for Shift_JIS and U for UTF-8,
  # followed by the search string. Up to 99 example sentences may be
  # displayed.
  KEYS = { english: 'E', common: 'P', exact: 'Q', both: 'R',
           japanese: 'J', start_kanji: 'K', any_kanji: 'L', kanji: 'M',
           glossing: 'G', no_repeated: 'H', jis_kanji: 'H',
           multi_radical: 'X' }.freeze

  # KANJIDIC letter codes (see
  # http://www.edrdg.org/kanjidic/kanjidic.html)
  # [U] the Unicode/ISO 10646 code of the kanji in hexadecimal;
  # [N] the index in Nelson (Modern Reader's Japanese-English Character
  # Dictionary)
  # [B] the classification radical number of the kanji (as in Nelson);
  # [C] the "classical" radical number (where this differs from the one
  # used in Nelson);
  # [S] the total stroke-count of the kanji;
  # [G] the "grade" of the kanji, In this case, G2 means it is a Jouyou
  # (general use) kanji taught in the second year of elementary
  # schooling in Japan;
  # [H] the index in Halpern (New Japanese-English Character
  # Dictionary);
  # [F] the rank-order frequency of occurrence of the kanji in Japanese;
  # [P] the "SKIP" coding of the kanji, as used in Halpern;
  # [K] the index in the Gakken Kanji Dictionary (A New Dictionary of
  # Kanji Usage);
  # [L] the index in Heisig (Remembering The Kanji);
  # [I] the index in the Spahn & Hadamitsky dictionary.
  # [Q] the Four-Corner code;
  # [MN,MP] the index and page number in the 13-volume Morohashi
  # "DaiKanWaJiten";
  # [E] the index in Henshall (A Guide To Remembering Japanese
  # Characters);
  # [Y] the PinYin (Chinese) pronunciation(s) of the kanji;
  KANJIDIC_CODES = { unicode: 'U', nelson: 'N', radical: 'B',
                     classical: 'C', stroke: 'S', grade: 'G',
                     halpern: 'H', frequency: 'F', skip: 'P',
                     gakken: 'K', heisig: 'L', spahn: 'I', corner: 'Q',
                     morohashi_index: 'MN', morohashi_page: 'MP',
                     henshall: 'E', pinyin: 'Y' }.freeze

  # Reference URI for Backdoor Entry/API
  URI_DEFAULT = 'http://www.edrdg.org/cgi-bin/wwwjdic/wwwjdic.cgi?'

  # Monash URI for Backdoor Entry/API
  URI_OLD = 'http://nihongo.monash.edu/cgi-bin/wwwjdic?'

  # URIs for Backdoor Entry/API
  URIS = { edrdg: URI_DEFAULT, monash: URI_OLD }.freeze
end
