# frozen_string_literal: true

#--
# wwwjdic
# © 2014-2021 Marco Bresciani
#
# This file is part of wwwjdic.
#
# wwwjdic is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# wwwjdic is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License
# along with wwwjdic. If not, see <http://www.gnu.org/licenses/>.
#
# SPDX-FileCopyrightText: 2014-2021 Marco Bresciani
#
# SPDX-License-Identifier: GPL-3.0-or-later
#++

module WWWJDic
  module Parsers
    # This class is an implementation of the Parsable duck type that
    # checks the display type. The only needed parameter is the word to
    # translate.
    #
    # t is the search type:
    # * for dictionary lookups use:
    #   * D where the lookup text is in ASCII, EUC, ISO-2022-JP or UCS
    # (the old uxxxx format);
    #   * S where the lookup text is in Shift-JIS;
    #   * U where the lookup text is in UTF-8.
    # * for kanji lookups use:
    #   * K for all lookups via codes, codepoints, etc. or where the
    # text string (e.g. kanji or reading) is in EUC;
    #   * M for all lookups using a text string in UTF8;
    #   * N for all lookups using a text string in Shift_JIS;
    # * for text glossing/translate words in text use:
    #   * G where the text is in EUC, ISO-2022-JP or UCS;
    #   * H where the text is in Shift-JIS;
    #   * I where the text is in UTF-8.
    # * for multi-radical kanji lookups use:
    #   * B where the text is in EUC, ISO-2022-JP or UCS;
    #   * C where the text is in Shift-JIS;
    #   * F where the text is in UTF-8.
    # * for example sentence lookups via indexed Japanese words, use E.
    # * for example sentence lookups using a regular expression, use T.
    #
    # Author:: {Marco Bresciani}[mailto:marcobresciani_1974@libero.it]
    # Copyright:: © 2014-2021 Marco Bresciani
    # License:: GNU General Public License version 3
    class Search
      # The parsable duck type interface to every parser usage.
      def parse(value = '')
        return 'D' if value == CGI.escape(value).to_s

        'U'
      end
    end
  end
end
